#include <QApplication>
#include <QImage>
#include <QPainter>
#include <QPen>
#include <QColor>

#include <cstdio>
#include <iostream>
#include <fstream>
#include <cstdlib>

#define WIDTH 32
#define HEIGHT 32

using namespace std;

/* This generates all tiles, even thought we won't use all of them. */
char tiles[105][6] =
{
    " ", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P",
    "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z",
    "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p",
    "q", "r", "s", "t", "u", "v", "w", "x", "y", "z",
    "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
    ":", "/",
    "0_NW", "0_NE", "0_SW", "0_SE",
    "1_NW", "1_NE", "1_SW", "1_SE",
    "2_NW", "2_NE", "2_SW", "2_SE",
    "3_NW", "3_NE", "3_SW", "3_SE",
    "4_NW", "4_NE", "4_SW", "4_SE",
    "5_NW", "5_NE", "5_SW", "5_SE",
    "6_NW", "6_NE", "6_SW", "6_SE",
    "7_NW", "7_NE", "7_SW", "7_SE",
    "8_NW", "8_NE", "8_SW", "8_SE",
    "9_NW", "9_NE", "9_SW", "9_SE",
};

/* Now we pick out the ones we'll actually use. Don't worry, NIOS2-IDE will compile out
 * the rest of the arrays in -O2 mode. */
char active_tiles[72][6] =
{
    "blank", "A", "F", "M", "P", "S", "T", "W",
    "a", "d", "e", "h", "i", "n", "o", "r", "s", "t", "u", "y",
    "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
    "colon", "slash",
    "0_NW", "0_NE", "0_SW", "0_SE",
    "1_NW", "1_NE", "1_SW", "1_SE",
    "2_NW", "2_NE", "2_SW", "2_SE",
    "3_NW", "3_NE", "3_SW", "3_SE",
    "4_NW", "4_NE", "4_SW", "4_SE",
    "5_NW", "5_NE", "5_SW", "5_SE",
    "6_NW", "6_NE", "6_SW", "6_SE",
    "7_NW", "7_NE", "7_SW", "7_SE",
    "8_NW", "8_NE", "8_SW", "8_SE",
    "9_NW", "9_NE", "9_SW", "9_SE",
};

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);
    char temp[1023];
    char title[1023];
    QImage qi(WIDTH, HEIGHT, QImage::Format_RGB888);
    QPainter qp(&qi);
    qp.setPen(QPen(Qt::white));
    qp.setFont(QFont("Bitstream Vera Mono", 20, QFont::Bold));

    ofstream header;
    header.open("BitstreamVeraMono.h");

    for (int i = 0; i < 105; i++)
    {
        printf("Generating %s...\n", tiles[i]);
        qi.fill(0);
        if (i < 65) qp.drawText(0, 0, WIDTH, HEIGHT, Qt::AlignCenter, tiles[i]);
        else
        {
            qp.setFont(QFont("Bitstream Vera Mono", 56, QFont::Bold));
            char number[1024];
            sprintf(number, "%c", tiles[i][0]);

            int x, y;
            x = ((i-1) % 2 == 1) ? -32 : 0;
            y = (((i-1) / 2) % 2 == 1) ? -32 : 0;

            qp.drawText(x, y, WIDTH * 2, HEIGHT * 2, Qt::AlignCenter, number);
        }

        if (i == 0) sprintf(title, "bvm_%s", "blank");
        else if (i == 63) sprintf(title, "bvm_%s", "colon");
        else if (i == 64) sprintf(title, "bvm_%s", "slash");
        else sprintf(title, "bvm_%s", tiles[i]);

        sprintf(temp, "%s.bmp", title);
        qi.save(temp);

        ofstream hout;
        sprintf(temp, "%s.h", title);
        hout.open(temp);

        sprintf(temp, "unsigned short %s[%d*%d/4] = {", title, WIDTH, HEIGHT);
        hout << temp << endl;

        for (int row = 0; row < HEIGHT; row++)
        {
            for (int col = 0; col < (WIDTH/4); col++)
            {
                unsigned int one = (qBlue(qi.pixel(col*4 + 0, row)) >> 4) & 0xf;
                unsigned int two = (qBlue(qi.pixel(col*4 + 1, row)) >> 4) & 0xf;
                unsigned int thr = (qBlue(qi.pixel(col*4 + 2, row)) >> 4) & 0xf;
                unsigned int fou = (qBlue(qi.pixel(col*4 + 3, row)) >> 4) & 0xf;

                sprintf(temp, "\t0x%04x,", fou | (thr << 4) | (two << 8) | (one << 12));
                hout << temp << endl;
            }
        }

        hout << "};" << endl;
        hout.close();

        sprintf(temp, "#include \"BitstreamVeraMono/%s.h\"", title);
        header << temp << endl;
    }

    header << endl << "unsigned short *tile_data[72] =" << endl << "{" << endl;
    for(int i = 0; i < 72; i++)
       header << "\tbvm_" << active_tiles[i] << "," << endl;
    header << "};" << endl << endl;

    for(int i = 0; i < 72; i++)
       header << "#define TILE_" << active_tiles[i] << " (" << i << ")" << endl;

    header.close();
}
